@@ -52,7 +52,7 @@ static int qgroup_assign(int assign, int argc, char **argv)
 	/*
 	 * FIXME src should accept subvol path
 	 */
-	if ((args.src >> 48) >= (args.dst >> 48)) {
+	if (btrfs_qgroup_level(args.src) >= btrfs_qgroup_level(args.dst)) {
 		fprintf(stderr, "ERROR: bad relation requested '%s'\n", path);
 		return 1;
 	}
@@ -226,8 +226,7 @@ static const char * const cmd_qgroup_show_usage[] = {
 	"--gbytes       show sizes in GiB, or GB with --si",
 	"--tbytes       show sizes in TiB, or TB with --si",
 	"--sort=qgroupid,rfer,excl,max_rfer,max_excl",
-	"               list qgroups in order of qgroupid,"
-	"rfer,max_rfer or max_excl",
+	"               list qgroups sorted by specified items",
 	"               you can use '+' or '-' in front of each item.",
 	"               (+:ascending, -:descending, ascending default)",
 	NULL
@@ -252,9 +251,8 @@ static int cmd_qgroup_show(int argc, char **argv)
 	optind = 1;
 	while (1) {
 		int c;
-		int option_index = 0;
 		static const struct option long_options[] = {
-			{"sort", 1, NULL, 'S'},
+			{"sort", required_argument, NULL, 'S'},
 			{"raw", no_argument, NULL, GETOPT_VAL_RAW},
 			{"kbytes", no_argument, NULL, GETOPT_VAL_KBYTES},
 			{"mbytes", no_argument, NULL, GETOPT_VAL_MBYTES},
@@ -266,9 +264,8 @@ static int cmd_qgroup_show(int argc, char **argv)
 				GETOPT_VAL_HUMAN_READABLE},
 			{ NULL, 0, NULL, 0 }
 		};
-		c = getopt_long(argc, argv, "pcreFf",
-				long_options, &option_index);
 
+		c = getopt_long(argc, argv, "pcreFf", long_options, NULL);
 		if (c < 0)
 			break;
 		switch (c) {
